import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:mfsdk/model/MFError.dart';
import 'package:mfsdk/model/directpayment/MFCardInfo.dart';
import 'package:mfsdk/model/directpayment/SDKDirectPaymentResponse.dart';
import 'package:mfsdk/model/executepayment/MFExecutePaymentRequest.dart';
import 'package:mfsdk/model/executepayment/SDKExecutePaymentResponse.dart';
import 'package:mfsdk/model/MyBaseResponse.dart';
import 'package:mfsdk/model/initpayment/SDKInitiatePaymentResponse.dart';
import 'package:mfsdk/model/paymentstatus/MFPaymentStatusRequest.dart';
import 'package:mfsdk/model/paymentstatus/SDKPaymentStatusResponse.dart';
import 'package:mfsdk/views/MFResult.dart';
import 'package:mfsdk/views/APPCONST.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';

var MFSDK = new MFatoorahSDK();

class MFatoorahSDK implements CallBackPaymentStatus {

  Function func;
  BuildContext myContext;

  // Initiate Payment
  void initiatePayment(String apiLang, Map request, Function func) async{

    http.post(
        APPCONST.INITIATE_PAYMENT,
        headers: {
          HttpHeaders.authorizationHeader: APPCONST.TOKEN,
          HttpHeaders.acceptLanguageHeader: apiLang
        }, body: request).then((http.Response response) {

      final int statusCode = response.statusCode;

      if (statusCode < 200 || statusCode >= 400 || json == null) {
        var error = MyBaseResponse.fromJson(json.decode(response.body));
        func(MFResult.fail(new MFError(200, error.message)));
      }

      var result = SDKInitiatePaymentResponse
          .fromJson(json.decode(response.body)).data;
      print(result.toJson());
      func(MFResult.success(result));
    });

//    func((await MFSDKController().initiatePaymentAPI(apiLang, url, token, data)).data);
  }

  // Execute  Payment
  void executePayment(BuildContext context,
      MFExecutePaymentRequest request, String apiLang, Function func) async{

    if(request.callBackUrl == null || request.callBackUrl.isEmpty)
      request.callBackUrl = APPCONST.CALL_BACK_URL;

    if(request.errorUrl == null || request.errorUrl.isEmpty)
      request.errorUrl = APPCONST.ERROR_URL;

    http.post(
        APPCONST.EXECUTE_PAYMENT,
        headers: {
          HttpHeaders.authorizationHeader: APPCONST.TOKEN,
          HttpHeaders.acceptLanguageHeader: apiLang
        }, body: request.toJson()).then((http.Response response) {

      final int statusCode = response.statusCode;

      if (statusCode < 200 || statusCode >= 400 || json == null) {
        print("statusCode: " + statusCode.toString());
        var error = MyBaseResponse.fromJson(json.decode(response.body));
        func(MFResult.fail(new MFError(200, error.message)));
      }

      var result = SDKExecutePaymentResponse
          .fromJson(json.decode(response.body)).data;

      if (!result.isDirectPayment) {
        this.func = func;
        this.myContext = context;
        showWebView(result.paymentURL);
      }
      else
        func(MFResult.fail(new MFError(APPCONST.CONFIG_ERROR, APPCONST.incorrectPaymentMethod)));
    });
  }

  // Execute Direct Payment
  void executeDirectPayment(BuildContext context,
      MFExecutePaymentRequest request, MFCardInfo mfCardInfo, String apiLang,
      Function func) async{

    if(request.callBackUrl == null || request.callBackUrl.isEmpty)
      request.callBackUrl = APPCONST.CALL_BACK_URL;

    if(request.errorUrl == null || request.errorUrl.isEmpty)
      request.errorUrl = APPCONST.ERROR_URL;

    http.post(
        APPCONST.EXECUTE_PAYMENT,
        headers: {
          HttpHeaders.authorizationHeader: APPCONST.TOKEN,
          HttpHeaders.acceptLanguageHeader: apiLang
        }, body: request.toJson()).then((http.Response response) {

      final int statusCode = response.statusCode;

      if (statusCode < 200 || statusCode >= 400 || json == null) {
        print("statusCode: " + statusCode.toString());
        var error = MyBaseResponse.fromJson(json.decode(response.body));
        func(MFResult.fail(new MFError(200, error.message)));
      }

      var result = SDKExecutePaymentResponse
          .fromJson(json.decode(response.body)).data;

      if (result.isDirectPayment) {
        directPayment(context, apiLang, result.invoiceId, result.paymentURL, mfCardInfo, func);
      }
      else
        func(MFResult.fail(new MFError(APPCONST.CONFIG_ERROR, APPCONST.accountDoNotHaveDirectPayment)));
    });
  }

  // Direct Payment
  void directPayment(BuildContext context,
      String apiLang, int invoiceId, String paymentURL, MFCardInfo request, Function func) async{

    http.post(paymentURL,
        headers: {
          HttpHeaders.contentTypeHeader: ContentType.json.toString(),
          HttpHeaders.authorizationHeader: APPCONST.TOKEN,
          HttpHeaders.acceptLanguageHeader: apiLang
        }, body: jsonEncode(request.toJson())).then((http.Response response) {

      final int statusCode = response.statusCode;

      if (statusCode < 200 || statusCode >= 400 || json == null) {
        var error = MyBaseResponse.fromJson(json.decode(response.body));
        func(MFResult.fail(new MFError(200, error.message)));
      }

      var result = SDKDirectPaymentResponse
          .fromJson(json.decode(response.body)).data;

      print(result.toJson());

      if(request.bypass3DS) {
        var request = MFPaymentStatusRequest(paymentId: result.paymentId).toJson();
        paymentStatus(apiLang, request, func);
      }
      else {
        this.func = func;
        this.myContext = context;
        showWebView(result.paymentURL);
      }
    });
  }

  // Payment Status
  void paymentStatus(
      String apiLang, Map request, Function func) async{

    http.post(
        APPCONST.PAYMENT_STATUS, headers: {
      HttpHeaders.contentTypeHeader: ContentType.json.toString(),
      HttpHeaders.authorizationHeader: APPCONST.TOKEN,
      HttpHeaders.acceptLanguageHeader: apiLang
    }, body: jsonEncode(request)).then((http.Response response) {

      final int statusCode = response.statusCode;

      if (statusCode < 200 || statusCode >= 400 || json == null) {
        var error = MyBaseResponse.fromJson(json.decode(response.body));
        func(MFResult.fail(new MFError(200, error.message)));
      }

      var result = SDKPaymentStatusResponse
          .fromJson(json.decode(response.body)).data;

//      print(result.toJson());

      func(MFResult.success(result));
    });
  }

  void showWebView(String paymentURL) async{
//    runApp(new MyApp(paymentURL, this));
    Navigator.push(myContext,
      MaterialPageRoute(builder: (context) => MyApp(paymentURL, this)),
    );
  }

  @override
  void callBackPaymentStatus(MFResult result) {
    print("callBackPaymentStatus: ");
    if(func != null) {
      func(result);
      Navigator.pop(myContext);
    }
  }
}

abstract class CallBackPaymentStatus {
  void callBackPaymentStatus(MFResult result);
}

class MyApp extends StatefulWidget {

  var paymentURL = "";
  CallBackPaymentStatus callBackPaymentStatus;
  MyApp(String paymentURL, CallBackPaymentStatus callBackPaymentStatus) {
    this.paymentURL = paymentURL;
    this.callBackPaymentStatus = callBackPaymentStatus;
  }

  @override
  _MyAppState createState() => new _MyAppState(paymentURL, callBackPaymentStatus);
}

class _MyAppState extends State<MyApp> {

    var paymentURL = "";
    CallBackPaymentStatus callBackPaymentStatus;
    _MyAppState(String paymentURL, CallBackPaymentStatus callBackPaymentStatus) {
    this.paymentURL = paymentURL;
    this.callBackPaymentStatus = callBackPaymentStatus;
  }

  InAppWebViewController webView;
  String url = "";
  double progress = 0;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('InAppWebView Example'),
        ),
        body: Container(
            child: Column(children: <Widget>[
              Container(
                padding: EdgeInsets.all(20.0),
                child: Text(
                    "CURRENT URL\n${(url.length > 50) ? url.substring(0, 50) + "..." : url}"),
              ),
              Container(
                  padding: EdgeInsets.all(10.0),
                  child: progress < 1.0
                      ? LinearProgressIndicator(value: progress)
                      : Container()),
              Expanded(
                child: Container(
                  margin: const EdgeInsets.all(10.0),
                  decoration:
                  BoxDecoration(border: Border.all(color: Colors.blueAccent)),
                  child: InAppWebView(
                    initialUrl: paymentURL,
                    initialHeaders: {},
                    initialOptions: InAppWebViewWidgetOptions(
                        crossPlatform: InAppWebViewOptions(
                          debuggingEnabled: true,
                        )
                    ),
                    onWebViewCreated: (InAppWebViewController controller) {
                      webView = controller;
                    },
                    onLoadStart: (InAppWebViewController controller, String url) {
                      setState(() {
                        if(url.contains(APPCONST.CALL_BACK_URL) || url.contains(APPCONST.ERROR_URL)) {
                          var request = MFPaymentStatusRequest(paymentId:
                          Uri.parse(url).queryParameters["paymentId"]).toJson();
                          MFSDK.paymentStatus("en", request, (MFResult result) => {
                            callBackPaymentStatus.callBackPaymentStatus(result)
                          });
                        }
                        this.url = url;
                      });
                    },
                    onLoadStop: (InAppWebViewController controller, String url) async {
                      setState(() {
                        this.url = url;
                      });
                    },
                    onProgressChanged: (InAppWebViewController controller, int progress) {
                      setState(() {
                        this.progress = progress / 100;
                      });
                    },
                  ),
                ),
              ),
              ButtonBar(
                alignment: MainAxisAlignment.center,
                children: <Widget>[
                  RaisedButton(
                    child: Icon(Icons.arrow_back),
                    onPressed: () {
                      if (webView != null) {
                        webView.goBack();
                      }
                    },
                  ),
                  RaisedButton(
                    child: Icon(Icons.arrow_forward),
                    onPressed: () {
                      if (webView != null) {
                        webView.goForward();
                      }
                    },
                  ),
                  RaisedButton(
                    child: Icon(Icons.refresh),
                    onPressed: () {
                      if (webView != null) {
                        webView.reload();
                      }
                    },
                  ),
                ],
              ),
            ])),
      ),
    );
  }
}

//class WebViewExample extends StatefulWidget {
//  var paymentURL = "";
//  WebViewExample(String paymentURL) {
//    this.paymentURL = paymentURL;
//  }
//
//  @override
//  _WebViewExampleState createState() => _WebViewExampleState(paymentURL);
//}

//const String kNavigationExamplePage = '''
//<!DOCTYPE html><html>
//<head><title>Navigation Delegate Example</title></head>
//<body>
//<p>
//The navigation delegate is set to block navigation to the youtube website.
//</p>
//<ul>
//<ul><a href="https://www.youtube.com/">https://www.youtube.com/</a></ul>
//<ul><a href="https://www.google.com/">https://www.google.com/</a></ul>
//</ul>
//</body>
//</html>
//''';


//class _WebViewExampleState extends State<WebViewExample> {
//  final Completer<WebViewController> _controller =
//  Completer<WebViewController>();
//
//  var paymentURL = "";
//  _WebViewExampleState(String paymentURL) {
//    this.paymentURL = paymentURL;
//  }
//
//  @override
//  Widget build(BuildContext context) {
//    return Scaffold(
//      appBar: AppBar(
//        title: const Text('Flutter WebView example'),
//        // This drop down menu demonstrates that Flutter widgets can be shown over the web view.
//        actions: <Widget>[
//          NavigationControls(_controller.future),
//          SampleMenu(_controller.future),
//        ],
//      ),
//      // We're using a Builder here so we have a context that is below the Scaffold
//      // to allow calling Scaffold.of(context) so we can show a snackbar.
//      body: Builder(builder: (BuildContext context) {
//        return WebView(
//          initialUrl: paymentURL,
//          javascriptMode: JavascriptMode.unrestricted,
//          onWebViewCreated: (WebViewController webViewController) {
//            _controller.complete(webViewController);
//          },
//          // TODO(iskakaushik): Remove this when collection literals makes it to stable.
//          // ignore: prefer_collection_literals
//          javascriptChannels: <JavascriptChannel>[
//            _toasterJavascriptChannel(context),
//          ].toSet(),
//          navigationDelegate: (NavigationRequest request) {
////            if (request.url.startsWith('https://www.youtube.com/')) {
////              print('blocking navigation to $request}');
////              return NavigationDecision.prevent;
////            }
//            print('allowing navigation to $request');
//            return NavigationDecision.navigate;
//          },
//          onPageStarted: (String url) {
//            print('Page started loading: $url');
//            if(url.contains(APPCONST.CALL_BACK_URL) || url.contains(APPCONST.ERROR_URL)) {
//              var request = MFPaymentStatusRequest(paymentId:
//              Uri.parse(url).queryParameters["paymentId"]).toJson();
//              MFSDK.paymentStatus("en", request).then((result) => {
//                if(result.isSuccess())
//                  print(result.response.toJson())
//                else
//                  print(result.error.message)
//              });
//            }
//          },
//          onPageFinished: (String url) {
//            print('Page finished loading: $url');
//          },
//          gestureNavigationEnabled: true,
//        );
//      }),
////      floatingActionButton: favoriteButton(),
//    );
//  }
//
//  JavascriptChannel _toasterJavascriptChannel(BuildContext context) {
//    return JavascriptChannel(
//        name: 'Toaster',
//        onMessageReceived: (JavascriptMessage message) {
//          Scaffold.of(context).showSnackBar(
//            SnackBar(content: Text(message.message)),
//          );
//        });
//  }
//
////  Widget favoriteButton() {
////    return FutureBuilder<WebViewController>(
////        future: _controller.future,
////        builder: (BuildContext context,
////            AsyncSnapshot<WebViewController> controller) {
////          if (controller.hasData) {
////            return FloatingActionButton(
////              onPressed: () async {
////                final String url = await controller.data.currentUrl();
////                Scaffold.of(context).showSnackBar(
////                  SnackBar(content: Text('Favorited $url')),
////                );
////              },
////              child: const Icon(Icons.favorite),
////            );
////          }
////          return Container();
////        });
////  }
//}
//
//enum MenuOptions {
//  showUserAgent,
//  listCookies,
//  clearCookies,
//  addToCache,
//  listCache,
//  clearCache,
//  navigationDelegate,
//}
//
//class SampleMenu extends StatelessWidget {
//  SampleMenu(this.controller);
//
//  final Future<WebViewController> controller;
//  final CookieManager cookieManager = CookieManager();
//
//  @override
//  Widget build(BuildContext context) {
//    return FutureBuilder<WebViewController>(
//      future: controller,
//      builder:
//          (BuildContext context, AsyncSnapshot<WebViewController> controller) {
//        return PopupMenuButton<MenuOptions>(
//          onSelected: (MenuOptions value) {
//            switch (value) {
//              case MenuOptions.showUserAgent:
//                _onShowUserAgent(controller.data, context);
//                break;
//              case MenuOptions.listCookies:
//                _onListCookies(controller.data, context);
//                break;
//              case MenuOptions.clearCookies:
//                _onClearCookies(context);
//                break;
//              case MenuOptions.addToCache:
//                _onAddToCache(controller.data, context);
//                break;
//              case MenuOptions.listCache:
//                _onListCache(controller.data, context);
//                break;
//              case MenuOptions.clearCache:
//                _onClearCache(controller.data, context);
//                break;
//              case MenuOptions.navigationDelegate:
//                _onNavigationDelegateExample(controller.data, context);
//                break;
//            }
//          },
//          itemBuilder: (BuildContext context) => <PopupMenuItem<MenuOptions>>[
//            PopupMenuItem<MenuOptions>(
//              value: MenuOptions.showUserAgent,
//              child: const Text('Show user agent'),
//              enabled: controller.hasData,
//            ),
//            const PopupMenuItem<MenuOptions>(
//              value: MenuOptions.listCookies,
//              child: Text('List cookies'),
//            ),
//            const PopupMenuItem<MenuOptions>(
//              value: MenuOptions.clearCookies,
//              child: Text('Clear cookies'),
//            ),
//            const PopupMenuItem<MenuOptions>(
//              value: MenuOptions.addToCache,
//              child: Text('Add to cache'),
//            ),
//            const PopupMenuItem<MenuOptions>(
//              value: MenuOptions.listCache,
//              child: Text('List cache'),
//            ),
//            const PopupMenuItem<MenuOptions>(
//              value: MenuOptions.clearCache,
//              child: Text('Clear cache'),
//            ),
//            const PopupMenuItem<MenuOptions>(
//              value: MenuOptions.navigationDelegate,
//              child: Text('Navigation Delegate example'),
//            ),
//          ],
//        );
//      },
//    );
//  }
//
//  void _onShowUserAgent(
//      WebViewController controller, BuildContext context) async {
//    // Send a message with the user agent string to the Toaster JavaScript channel we registered
//    // with the WebView.
//    await controller.evaluateJavascript(
//        'Toaster.postMessage("User Agent: " + navigator.userAgent);');
//  }
//
//  void _onListCookies(
//      WebViewController controller, BuildContext context) async {
//    final String cookies =
//    await controller.evaluateJavascript('document.cookie');
//    Scaffold.of(context).showSnackBar(SnackBar(
//      content: Column(
//        mainAxisAlignment: MainAxisAlignment.end,
//        mainAxisSize: MainAxisSize.min,
//        children: <Widget>[
//          const Text('Cookies:'),
//          _getCookieList(cookies),
//        ],
//      ),
//    ));
//  }
//
//  void _onAddToCache(WebViewController controller, BuildContext context) async {
//    await controller.evaluateJavascript(
//        'caches.open("test_caches_entry"); localStorage["test_localStorage"] = "dummy_entry";');
//    Scaffold.of(context).showSnackBar(const SnackBar(
//      content: Text('Added a test entry to cache.'),
//    ));
//  }
//
//  void _onListCache(WebViewController controller, BuildContext context) async {
//    await controller.evaluateJavascript('caches.keys()'
//        '.then((cacheKeys) => JSON.stringify({"cacheKeys" : cacheKeys, "localStorage" : localStorage}))'
//        '.then((caches) => Toaster.postMessage(caches))');
//  }
//
//  void _onClearCache(WebViewController controller, BuildContext context) async {
//    await controller.clearCache();
//    Scaffold.of(context).showSnackBar(const SnackBar(
//      content: Text("Cache cleared."),
//    ));
//  }
//
//  void _onClearCookies(BuildContext context) async {
//    final bool hadCookies = await cookieManager.clearCookies();
//    String message = 'There were cookies. Now, they are gone!';
//    if (!hadCookies) {
//      message = 'There are no cookies.';
//    }
//    Scaffold.of(context).showSnackBar(SnackBar(
//      content: Text(message),
//    ));
//  }
//
//  void _onNavigationDelegateExample(
//      WebViewController controller, BuildContext context) async {
//    final String contentBase64 =
//    base64Encode(const Utf8Encoder().convert(kNavigationExamplePage));
//    await controller.loadUrl('data:text/html;base64,$contentBase64');
//  }
//
//  Widget _getCookieList(String cookies) {
//    if (cookies == null || cookies == '""') {
//      return Container();
//    }
//    final List<String> cookieList = cookies.split(';');
//    final Iterable<Text> cookieWidgets =
//    cookieList.map((String cookie) => Text(cookie));
//    return Column(
//      mainAxisAlignment: MainAxisAlignment.end,
//      mainAxisSize: MainAxisSize.min,
//      children: cookieWidgets.toList(),
//    );
//  }
//}
//
//class NavigationControls extends StatelessWidget {
//  const NavigationControls(this._webViewControllerFuture)
//      : assert(_webViewControllerFuture != null);
//
//  final Future<WebViewController> _webViewControllerFuture;
//
//  @override
//  Widget build(BuildContext context) {
//    return FutureBuilder<WebViewController>(
//      future: _webViewControllerFuture,
//      builder:
//          (BuildContext context, AsyncSnapshot<WebViewController> snapshot) {
//        final bool webViewReady =
//            snapshot.connectionState == ConnectionState.done;
//        final WebViewController controller = snapshot.data;
//        return Row(
//          children: <Widget>[
//            IconButton(
//              icon: const Icon(Icons.arrow_back_ios),
//              onPressed: !webViewReady
//                  ? null
//                  : () async {
//                if (await controller.canGoBack()) {
//                  await controller.goBack();
//                } else {
//                  Scaffold.of(context).showSnackBar(
//                    const SnackBar(content: Text("No back history item")),
//                  );
//                  return;
//                }
//              },
//            ),
//            IconButton(
//              icon: const Icon(Icons.arrow_forward_ios),
//              onPressed: !webViewReady
//                  ? null
//                  : () async {
//                if (await controller.canGoForward()) {
//                  await controller.goForward();
//                } else {
//                  Scaffold.of(context).showSnackBar(
//                    const SnackBar(
//                        content: Text("No forward history item")),
//                  );
//                  return;
//                }
//              },
//            ),
//            IconButton(
//              icon: const Icon(Icons.replay),
//              onPressed: !webViewReady
//                  ? null
//                  : () {
//                controller.reload();
//              },
//            ),
//          ],
//        );
//      },
//    );
//  }
//}

